from typing import Union

from coincurve import PublicKey
from sha3 import keccak_256 as _keccak_256

__all__ = ("get_eth_addr", "to_checksum_addr")


def keccak_256(b: bytes) -> bytes:
    h = _keccak_256()
    h.update(b)
    return h.digest()


def to_checksum_addr(eth_addr: str) -> str:
    """
    Convert eth address to eth checksum address.

    EIP 55: https://github.com/ethereum/EIPs/blob/master/EIPS/eip-55.md#implementation
    """
    address = eth_addr.lower().replace("0x", "")
    addr_hash = keccak_256(address.encode()).hex()

    res = []
    for a, h in zip(address, addr_hash):
        if int(h, 16) >= 8:
            res.append(a.upper())
        else:
            res.append(a)

    return "0x" + "".join(res)


def get_eth_addr(pk: Union[str, bytes]) -> str:
    """Get ETH address from a public key."""
    pk_bytes = bytes.fromhex(pk) if isinstance(pk, str) else pk

    if len(pk_bytes) != 64:
        pk_bytes = PublicKey(pk_bytes).format(False)[1:]

    return to_checksum_addr(f"0x{keccak_256(pk_bytes)[-20:].hex()}")



from typing import Tuple
from bip32 import HARDENED_INDEX


__all__ = ("COIN_PATHS",)


def coin_path_by_index(index: int = 0) -> Tuple[int, int]:
    # Full list is at https://github.com/satoshilabs/slips/blob/master/slip-0044.md#registered-coin-types
    return (44 + HARDENED_INDEX, index + HARDENED_INDEX)


COIN_PATHS = {
    "BTC": coin_path_by_index(),
    "TESTNET": coin_path_by_index(1),
    "LTC": coin_path_by_index(2),
    "DOGE": coin_path_by_index(3),
    "ETH": coin_path_by_index(60),
    "ETC": coin_path_by_index(61),
}



from typing import Tuple, Iterable, Union

from bip32 import BIP32, HARDENED_INDEX
from mnemonic import Mnemonic

from bip44.consts import COIN_PATHS, coin_path_by_index

__all__ = ("Wallet",)


class Wallet:
    def __init__(self, mnemonic: str, language: str = "english", passphrase: str = ""):
        """
        BIP44 HD wallet with a master mnemonic.

        :param mnemonic (str): The master mnemonic to derive keys
        :param language (str, optional): The mnemonic's language, default: "english"
        :param passphrase (str, optional): The mnemonic's passphrase, default: ""
        """
        self._seed = Mnemonic(language).to_seed(mnemonic, passphrase)
        self._bip32 = BIP32.from_seed(self._seed)

    def _derive_secret(self, path: Union[str, Iterable[int]]) -> bytes:
        return self._bip32.get_privkey_from_path(path)

    def _derive_public(self, path: Union[str, Iterable[int]]) -> bytes:
        return self._bip32.get_pubkey_from_path(path)

    def derive_secret_key(self, path: str) -> bytes:
        """
        Derive secret key from path.

        Args:
            :param path (str): Key path, e.g. m/44'/0'/0'/0/0

        Returns:
            bytes: Bytes of secret/private key
        """
        return self._derive_secret(path)

    def derive_public_key(self, path: str) -> bytes:
        """
        Derive public key from path.

        Args:
            :param path (str): Key path, e.g. m/44'/0'/0'/0/0

        Returns:
            bytes: Bytes of public key, compressed format
        """
        return self._derive_public(path)

    def derive_account(
        self,
        coin: Union[str, int],
        account: int = 0,
        change: int = 0,
        address_index: int = 0,
    ) -> Tuple[bytes, bytes]:
        """
        Derive secret and public key of account, following BIP44 standard like `m / purpose' / coin_type' / account' / change / address_index`.

        Args:
            :param coin (Union[str, int]): e.g. BTC or ETH, respectively, 0 or 60 in index
            :param account (int, optional): Account index, each user should have one
            :param change (int, optional): 0 for receiving payments and 1 for receiving changes, see BIP44 for details
            :param address_index (int, optional): Starting from 0

        Returns:
            Tuple[bytes, bytes]: Secret key and public key
        """
        if isinstance(coin, str):
            coin_path = COIN_PATHS[coin.upper()]
        else:
            coin_path = coin_path_by_index(coin)

        account_path = (account + HARDENED_INDEX, change, address_index)
        path = coin_path + account_path
        return (self._derive_secret(path), self._derive_public(path))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bip44']

package_data = \
{'': ['*']}

install_requires = \
['bip32>=0.0.8,<0.1.1', 'mnemonic>=0.19,<0.20', 'pysha3>=1.0.2,<2.0.0']

setup_kwargs = {
    'name': 'bip44',
    'version': '0.1.0',
    'description': 'Simple Python bip44 implementation. Mnemonic + bip32.',
    'long_description': '# python-bip44\n\n[![Codacy Badge](https://app.codacy.com/project/badge/Grade/4e4aa71f4a234dca809d014b0b214220)](https://www.codacy.com/manual/kigawas/python-bip44)\n[![CI](https://img.shields.io/github/workflow/status/kigawas/python-bip44/Build)](https://github.com/kigawas/python-bip44/actions)\n[![Codecov](https://img.shields.io/codecov/c/github/kigawas/python-bip44.svg)](https://codecov.io/gh/kigawas/python-bip44)\n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/bip44.svg)](https://pypi.org/project/bip44/)\n[![PyPI](https://img.shields.io/pypi/v/bip44.svg)](https://pypi.org/project/bip44/)\n[![License](https://img.shields.io/github/license/kigawas/python-bip44.svg)](https://github.com/kigawas/python-bip44)\n\nSimple Python [bip44](https://github.com/bitcoin/bips/blob/master/bip-0044.mediawiki) implementation. [Mnemonic](https://github.com/trezor/python-mnemonic) + [bip32](https://github.com/darosior/python-bip32).\n\n## Install\n\n`pip install bip44`\n\n## Quick Start\n\n```python\n>>> from coincurve import PrivateKey\n>>> from bip44 import Wallet\n>>> from bip44.utils import get_eth_addr\n>>> mnemonic = "purity tunnel grid error scout long fruit false embody caught skin gate"\n>>> w = Wallet(mnemonic)\n>>> sk, pk = w.derive_account("eth", account=0)\n>>> sk = PrivateKey(sk)\n>>> sk.public_key.format() == pk\nTrue\n>>> get_eth_addr(pk)\n\'0x7aD23D6eD9a1D98E240988BED0d78e8C81Ec296C\'\n```\n',
    'author': 'Weiliang Li',
    'author_email': 'to.be.impressive@gmail.com',
    'maintainer': 'Weiliang Li',
    'maintainer_email': 'to.be.impressive@gmail.com',
    'url': 'https://github.com/kigawas/python-bip44',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
